'' =================================================================================================
''
''   File....... anemometer_test.spin
''   Purpose.... 
''   Author..... Jon "JonnyMac" McPhalen (aka Jon Williams)
''               Copyright (c) 2010 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started.... 
''   Updated.... 08 MAY 2010
''
'' =================================================================================================


CON

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000

  CLK_FREQ = ((_clkmode - xtal1) >> 6) * _xinfreq
  MS_001   = CLK_FREQ / 1_000


CON

  #1, HOME, #8, BKSP, TAB, LF, CLREOL, CLRDN, CR, #16, CLS      ' PST formmatting control


OBJ

  term : "fullduplexserial"
  wind : "anemometer"


VAR

  long  wpulses
  

PUB main

  term.start(31, 30, %0000, 115_200)
  pause(1)
  term.tx(CLS)

  wind.init(0, @wpulses)                                        ' start anemometer monitor

  init(0, 50)                                                   ' start anemometer simulator

  repeat
    term.tx(HOME)
    term.dec(wpulses)
    term.tx(CLREOL)
    pause(500)
  

PUB pause(ms) | t

  t := cnt                                                      ' sync with system counter
  repeat ms
    waitcnt(t += MS_001)


PUB init(pin, fr)

  mask   := |< pin                                              ' pin mask for output
  period := clkfreq / fr
  ontix  := period >> 3                                         ' "bounce" 1/8th period
  offtix := (ontix << 2) + (ontix << 1) + ontix                 ' on * 7
  
  cognew(@breezy, 0)                                            ' start simulation                                          


DAT

                        org     0

breezy                  mov     dira, mask
                        mov     t0, cnt                         ' create sync point
{                       
pinon                   add     t0, ontix                       ' set on timing
                        mov     outa, mask                      ' clean output                     
                        waitcnt t0, #0
                        jmp     #pinoff
}

pinon                   xor     outa, mask                      ' toggle pin for "noisy" output
                        mov     t1, #500                        ' let it settle
:loop                   nop
                        djnz    t1, #:loop                   
                        mov     t1, cnt                         ' check noisy burst timing                        
                        subs    t1, t0
                        abs     t1, t1
                        cmp     t1, ontix               wc, wz
        if_b            jmp     #pinon
                        add     t0, ontix                       ' account for "on" timing                           


pinoff                  add     t0, offtix                      ' set "off" timing
                        mov     outa, #0                        ' output off
                        waitcnt t0, #0
                        jmp     #pinon                         
        
' --------------------------------------------------------------------------------------------------

mask                    long    0-0
period                  long    0-0
ontix                   long    0-0
offtix                  long    0-0

t0                      res     1
t1                      res     1

                        fit     492  

                        
DAT

{{

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}  